from setuptools import setup, find_packages

setup(
    name='detrain',
    version='0.2.5',
    author='A2N Finance', 
    author_email='levi@a2n.finance',
    packages=find_packages(),
    long_description=open('README.md').read()
)